<?php

declare(strict_types=1);

namespace App\Controller\OgBoot;

use App\Service\OgBoot\ConfigService;
use App\Service\OgBoot\StatusService;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Attribute\AsController;
use Symfony\Component\Routing\Attribute\Route;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

#[Route('/og-boot')]
#[AsController]
class OgBootController extends AbstractController
{
    public function __construct(
        private readonly StatusService $ogbootStatusService,
        private readonly ConfigService $ogbootConfigService,
    )
    {
    }

    /**
     * @throws TransportExceptionInterface
     * @throws ServerExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ClientExceptionInterface
     */
    #[Route('/status', name: 'ogboot_status',  methods: ['GET'])]
    public function status(): JsonResponse
    {
        $data = $this->ogbootStatusService->__invoke();

        return new JsonResponse( data: $data, status: Response::HTTP_OK);
    }


    /**
     * @throws TransportExceptionInterface
     * @throws ServerExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ClientExceptionInterface
     */
    #[Route('/config', name: 'ogboot_config',  methods: ['GET'])]
    public function config(): JsonResponse
    {
        $data = $this->ogbootConfigService->__invoke();

        return new JsonResponse( data: $data, status: Response::HTTP_OK);
    }
}
