<?php

namespace App\Controller\OgBoot\OgLive;

use App\Controller\OgBoot\AbstractOgBootController;
use App\Entity\OgLive;
use App\Model\OgLiveStatus;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Attribute\AsController;
use Symfony\Component\Validator\Exception\ValidatorException;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

#[AsController]
class SyncAction extends AbstractOgBootController
{
    /**
     * @throws TransportExceptionInterface
     * @throws ServerExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ClientExceptionInterface
     */
    public function __invoke(HttpClientInterface $httpClient, EntityManagerInterface $entityManager): JsonResponse
    {
        $content = $this->createRequest($httpClient, 'GET', $this->ogBootApiUrl . '/ogboot/v1/oglives');

        foreach ($content['installed_ogLives'] as $ogLive) {
            $ogLiveEntity = $this->entityManager->getRepository(OgLive::class)->findOneBy(['checksum' => $ogLive['id']]);
            if ($ogLiveEntity) {
                $this->extracted($ogLiveEntity, $ogLive);
                $this->entityManager->persist($ogLiveEntity);
            } else {
                $ogLiveEntity = new OgLive();
                $this->extracted($ogLiveEntity, $ogLive);
            }
            $this->entityManager->persist($ogLiveEntity);
        }
        $this->entityManager->flush();
        $this->serDefaultOgLive($content['default_oglive']);

        return new JsonResponse(data: $content, status: Response::HTTP_OK);
    }

    /**
     * @param OgLive|null $ogLiveEntity
     * @param mixed $ogLive
     * @return void
     */
    private function extracted(OgLive|null $ogLiveEntity, mixed $ogLive): void
    {
        $ogLiveEntity->setName($ogLive['filename']);
        $ogLiveEntity->setInstalled(true);
        $ogLiveEntity->setArchitecture($ogLive['architecture']);
        $ogLiveEntity->setDistribution($ogLive['distribution']);
        $ogLiveEntity->setFilename($ogLive['filename']);
        $ogLiveEntity->setKernel($ogLive['kernel']);
        $ogLiveEntity->setRevision($ogLive['revision']);
        $ogLiveEntity->setDirectory($ogLive['directory']);
        $ogLiveEntity->setChecksum($ogLive['id']);
        $ogLiveEntity->setStatus(OgLiveStatus::ACTIVE);
    }

    private function serDefaultOgLive(string $defaultOgLive): void
    {
        $ogLiveEntity = $this->entityManager->getRepository(OgLive::class)->findOneBy(['name' => $defaultOgLive]);

        if (!$ogLiveEntity) {
            return;
        }

        $ogLiveEntity->setIsDefault(true);
        $this->entityManager->persist($ogLiveEntity);
        $this->entityManager->flush();
    }
}