<?php

namespace App\Controller\OgBoot\PxeTemplate;

use App\Controller\OgBoot\AbstractOgBootController;
use App\Entity\PxeTemplate;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Attribute\AsController;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

#[AsController]
class PostAction extends AbstractOgBootController
{
    /**
     * @throws TransportExceptionInterface
     * @throws ServerExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ClientExceptionInterface
     */
    public function __invoke(PxeTemplate $data, HttpClientInterface $httpClient, EntityManagerInterface $entityManager): JsonResponse
    {
        try {
            $response = $httpClient->request('POST', $this->ogBootApiUrl.'/ogboot/v1/pxe-templates', [
                'headers' => [
                    'accept' => 'application/json',
                    'Content-Type' => 'application/json',
                ],
                'json' => [
                    'name_template' => $data->getName(),
                    'content_template' => $data->getTemplateContent(),
                ],
            ]);

            if ($response->getStatusCode() === Response::HTTP_OK) {
                $data->setSynchronized(true);
                $entityManager->persist($data);
                $entityManager->flush();
            }

            $data = json_decode($response->getContent(), true);
            return new JsonResponse($data, Response::HTTP_OK);

        } catch (ClientExceptionInterface $e) {
            $errorResponse = $e->getResponse();
            $errorContent = $errorResponse ? $errorResponse->getContent(false) : 'Client error occurred';
            return new JsonResponse(['error' => $errorContent], Response::HTTP_BAD_REQUEST);
        } catch (ServerExceptionInterface $e) {
            $errorResponse = $e->getResponse();
            $errorContent = $errorResponse ? $errorResponse->getContent(false) : 'Server error occurred';
            return new JsonResponse(['error' => $errorContent], Response::HTTP_INTERNAL_SERVER_ERROR);
        } catch (TransportExceptionInterface $e) {
            return new JsonResponse(['error' => 'Transport error occurred'], Response::HTTP_INTERNAL_SERVER_ERROR);
        }
    }
}