<?php

namespace App\Controller\OgDhcp\Subnet;

use App\Controller\OgDhcp\AbstractOgDhcpController;
use App\Dto\Input\SubnetAddSingleHostInput;
use App\Entity\Client;
use App\Entity\Subnet;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Attribute\AsController;
use Symfony\Component\Validator\Exception\ValidatorException;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

#[AsController]
class AddSingleHostAction extends AbstractOgDhcpController
{
    /**
     * @throws TransportExceptionInterface
     * @throws ServerExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ClientExceptionInterface
     */
    public function __invoke(SubnetAddSingleHostInput $input, Subnet $subnet, HttpClientInterface $httpClient): JsonResponse
    {
        $client = $input->client;
        /** @var Client $clientEntity */
        $clientEntity = $client->getEntity();

        $params = [
            'json' => [
                'host' => $clientEntity->getName(),
                'macAddress' => $clientEntity->getMac(),
                'address' => $clientEntity->getIp(),
            ]
        ];

        $content = $this->createRequest($httpClient, 'POST', $this->ogDhcpApiUrl.'/ogdhcp/v1/subnets/'.$subnet->getId().'/hosts', $params);

        if ($content->getStatusCode() === 200) {
            $subnet->addClient($clientEntity);
            $this->entityManager->persist($subnet);
            $this->entityManager->flush();
        }

        return new JsonResponse(data: $content, status: Response::HTTP_OK);
    }
}