<?php

namespace App\Controller\OgDhcp\Subnet;

use App\Controller\OgDhcp\AbstractOgDhcpController;
use App\Dto\Input\SubnetAddSingleHostInput;
use App\Dto\Input\SubnetAddSingleOrganizationalUnitInput;
use App\Entity\Client;
use App\Entity\OrganizationalUnit;
use App\Entity\Subnet;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Attribute\AsController;
use Symfony\Component\Validator\Exception\ValidatorException;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

#[AsController]
class AddSingleOrganizationalUnitAction extends AbstractOgDhcpController
{
    /**
     * @throws TransportExceptionInterface
     * @throws ServerExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ClientExceptionInterface
     */
    public function __invoke(SubnetAddSingleOrganizationalUnitInput $input, Subnet $subnet, HttpClientInterface $httpClient): JsonResponse
    {
        $ou = $input->organizationalUnitOutput;
        /** @var OrganizationalUnit $organizationalUnitEntity */
        $organizationalUnitEntity = $ou->getEntity();

        $params = [
            'json' => [
                'name' => $organizationalUnitEntity->getId(),
                'nextServer' => $organizationalUnitEntity->getNetworkSettings()?->getNextServer(),
                'bootFileName' => $organizationalUnitEntity->getNetworkSettings()?->getBootFileName(),
            ]
        ];

        $content = $this->createRequest($httpClient, 'POST', $this->ogDhcpApiUrl.'/ogdhcp/v1/subnets/'.$subnet->getId().'/classroom', $params);

        if ($content->getStatusCode() === 200) {
            $subnet->addOrganizationalUnit($organizationalUnitEntity);
            $this->entityManager->persist($subnet);
            $this->entityManager->flush();
        }

        return new JsonResponse(data: $content, status: Response::HTTP_OK);
    }
}