<?php

namespace App\Controller\OgDhcp\Subnet;

use App\Controller\OgDhcp\AbstractOgDhcpController;
use App\Dto\Input\SubnetAddHostInput;
use App\Dto\Output\ClientOutput;
use App\Entity\Client;
use App\Entity\Subnet;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Attribute\AsController;
use Symfony\Component\Validator\Exception\ValidatorException;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

#[AsController]
class PostHostAction extends AbstractOgDhcpController
{

    /**
     * @throws TransportExceptionInterface
     * @throws ServerExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ClientExceptionInterface
     */
    public function __invoke(SubnetAddHostInput $input, Subnet $subnet, HttpClientInterface $httpClient): JsonResponse
    {
        $clients = $input->clients;

        $subnet->setClients($clients);
        $this->entityManager->persist($subnet);
        $this->entityManager->flush();

        foreach ($clients as $client) {
            /** @var Client $clientEntity */
            $clientEntity = $client->getEntity();

            $data = [
                'host' => $clientEntity->getName(),
                'macAddress' => $clientEntity->getMac(),
                'address' => $clientEntity->getIp(),
            ];

            $params = [
                'json' => $data
            ];

            $content = $this->createRequest($httpClient, 'POST', $this->ogDhcpApiUrl.'/ogdhcp/v1/subnets/'.$subnet->getId().'/hosts', $params);
        }

        return new JsonResponse(status: Response::HTTP_OK);
    }
}