<?php

namespace App\Dto\Input;

use ApiPlatform\Metadata\ApiProperty;
use App\Dto\Output\ClientOutput;
use App\Dto\Output\CommandGroupOutput;
use App\Dto\Output\CommandOutput;
use App\Entity\CommandTask;
use App\Model\CommandTaskStatus;
use phpDocumentor\Reflection\Types\Boolean;
use Symfony\Component\Serializer\Annotation\Groups;
use Symfony\Component\Validator\Constraints as Assert;

final class CommandTaskInput
{
    /**
     * @var CommandOutput[]
     */
    #[Groups(['command-task:write'])]
    #[ApiProperty(
        description: 'Los comandos de la tarea',
        example: 'Comandos de la tarea',
    )]
    public array $commands = [];

    /**
     * @var CommandGroupOutput[]
     */
    #[Groups(['command-task:write'])]
    #[ApiProperty(
        description: 'Los grupos de comandos de la tarea',
        example: 'Grupos de comandos de la tarea',
    )]
    public array $commandGroups = [];

    /**
     * @var ClientOutput[]
     */
    #[Groups(['command-task:write'])]
    #[ApiProperty(
        description: 'Los clientes de la tarea',
        example: 'Clientes de la tarea',
    )]
    public array $clients = [];

    #[Assert\NotBlank(message: 'validators.command_task.datetime.not_blank')]
    #[Groups(['command-task:write'])]
    #[ApiProperty(
        description: 'La fecha y hora de la tarea',
        example: '2021-10-01T00:00:00+00:00',
    )]
    public ?\DateTimeInterface $dateTime = null;

    #[Groups(['command-task:write'])]
    #[ApiProperty(
        description: 'Los comentarios de la tarea',
        example: 'Comentarios de la tarea',
    )]
    public ?string $notes = null;

    public function __construct(?CommandTask $commandTask = null)
    {
        if (!$commandTask) {
            return;
        }

        if ($commandTask->getCommands()) {
            foreach ($commandTask->getCommands() as $command) {
                $this->commands[] = new CommandOutput($command);
            }
        }

        if ($commandTask->getCommandGroups()) {
            foreach ($commandTask->getCommandGroups() as $commandGroup) {
                $this->commandGroups[] = new CommandGroupOutput($commandGroup);
            }
        }

        if ($commandTask->getClients()) {
            foreach ($commandTask->getClients() as $client) {
                $this->clients[] = new ClientOutput($client);
            }
        }

        $this->dateTime = $commandTask->getDatetime();
        $this->notes = $commandTask->getNotes();
    }

    public function createOrUpdateEntity(?CommandTask $commandTask = null): CommandTask
    {
        if (!$commandTask) {
            $commandTask = new CommandTask();
        }

        foreach ($this->commands as $command) {
            $commandsToAdd[] = $command->getEntity();
        }

        $commandTask->setCommands( $commandsToAdd ?? [] );

        foreach ($this->commandGroups as $commandGroup) {
            $commandGroupsToAdd[] = $commandGroup->getEntity();
        }

        $commandTask->setCommandGroups( $commandGroupsToAdd ?? [] );

        foreach ($this->clients as $client) {
            $clientsToAdd[] = $client->getEntity();
        }

        $commandTask->setClients( $clientsToAdd ?? [] );

        $commandTask->setDatetime($this->dateTime);
        $commandTask->setStatus(CommandTaskStatus::PENDING);
        $commandTask->setNotes($this->notes);

        return $commandTask;
    }
}
