<?php

namespace App\Dto\Input;

use ApiPlatform\Metadata\ApiProperty;
use App\Dto\Output\ClientOutput;
use App\Dto\Output\PxeTemplateOutput;
use App\Entity\PxeBootFile;
use Symfony\Component\Serializer\Annotation\Groups;
use Symfony\Component\Validator\Constraints as Assert;

final class PxeBootFileInput
{
    #[Assert\NotNull(message: 'validators.pxe_boot_file.template.not_null')]
    #[Groups(['pxe-boot-file:write'])]
    #[ApiProperty(description: 'The pxeTemplate of the pxeBootFile', example: "PxeTemplate 1")]
    public ?PxeTemplateOutput $template = null;

    /**
     * @var ClientOutput[]
     */
    #[Groups(['pxe-boot-file:write'])]
    #[ApiProperty(description: 'The clients of the pxeBootFile', readableLink: false, writableLink: false, example: "Client 1")]
    public array $clients = [];

    public function __construct(?PxeBootFile $bootFile = null)
    {
        if (!$bootFile) {
            return;
        }

        $this->template = $bootFile->getTemplate();
        if ($bootFile->getClients()) {
            foreach ($bootFile->getClients() as $client) {
                $this->clients[] = new ClientOutput($client);
            }
        }
    }

    public function createOrUpdateEntity(?PxeBootFile $bootFile = null): PxeBootFile
    {
        if (!$bootFile) {
            $bootFile = new PxeBootFile();
        }

        $bootFile->setTemplate($this->template->getEntity());

        foreach ($this->clients as $client) {
            $clientsToAdd[] = $client->getEntity();
        }
        $bootFile->setClients( $clientsToAdd ?? [] );

        return $bootFile;
    }
}