<?php

namespace App\Dto\Output;

use ApiPlatform\Metadata\ApiProperty;
use ApiPlatform\Metadata\Get;
use App\Entity\Client;
use App\Entity\Partition;
use Symfony\Component\Serializer\Annotation\Groups;

#[Get(shortName: 'Client')]
final class ClientOutput extends AbstractOutput
{
    CONST string TYPE = 'client';

    #[Groups(['client:read', 'organizational-unit:read', 'pxe-boot-file:read', 'trace:read'])]
    public string $name;

    #[Groups(['client:read', 'organizational-unit:read'])]
    public string $type = self::TYPE;

    #[Groups(['client:read', 'organizational-unit:read', 'pxe-boot-file:read', 'trace:read'])]
    public ?string $ip = '';

    #[Groups(['client:read', 'organizational-unit:read', 'pxe-boot-file:read'])]
    public ?string $mac = '';

    #[Groups(['client:read', 'organizational-unit:read', 'trace:read'])]
    public ?string $serialNumber = '';

    #[Groups(['client:read'])]
    public ?string $netiface = '';

    #[Groups(['client:read'])]
    public ?string $netDriver = '';

    #[Groups(['client:read'])]
    #[ApiProperty(readableLink: true)]
    public ?OrganizationalUnitOutput $organizationalUnit = null;

    #[Groups(['client:read'])]
    public array $partitions = [];

    #[Groups(['client:read'])]
    public ?MenuOutput $menu = null;

    #[Groups(['client:read'])]
    #[ApiProperty(readableLink: true )]
    public ?HardwareProfileOutput $hardwareProfile = null;

    #[Groups(['client:read'])]
    #[ApiProperty(readableLink: true )]
    public ?OgLiveOutput $ogLive = null;

    #[Groups(['client:read'])]
    public ?array $position = ['x' => 0, 'y' => 0];

    #[Groups(['client:read'])]
    public ?string $status = '';

    #[Groups(['client:read'])]
    public \DateTime $createdAt;

    #[Groups(['client:read'])]
    public ?string $createdBy = null;

    public function __construct(Client $client)
    {
        parent::__construct($client);

        $this->name = $client->getName();
        $this->serialNumber = $client->getSerialNumber();
        $this->mac = $client->getMac();
        $this->ip = $client->getIp();
        $this->netiface = $client->getNetiface();
        $this->netDriver = $client->getNetDriver();

        if ($client->getOrganizationalUnit()) {
            $this->organizationalUnit = new OrganizationalUnitOutput($client->getOrganizationalUnit());
        }

        $this->partitions = $client->getPartitions()->map(
            fn(Partition $partition) => new PartitionOutput($partition)
        )->toArray();

        $this->menu = $client->getMenu() ? new MenuOutput($client->getMenu()) : null;
        $this->position = $client->getPosition();
        $this->hardwareProfile = $client->getHardwareProfile() ? new HardwareProfileOutput($client->getHardwareProfile()) : null;
        $this->ogLive = $client->getOgLive() ? new OgLiveOutput($client->getOgLive()) : null;
        $this->status = $client->getStatus();
        $this->createdAt = $client->getCreatedAt();
        $this->createdBy = $client->getCreatedBy();
    }
}