<?php

namespace App\Dto\Output;

use ApiPlatform\Metadata\Get;
use App\Entity\Client;
use App\Entity\Command;
use App\Entity\CommandGroup;
use App\Entity\CommandTask;
use Symfony\Component\Serializer\Annotation\Groups;

#[Get(shortName: 'CommandTask')]
final class CommandTaskOutput extends AbstractOutput
{
    #[Groups(['command-task:read'])]
    public array $commands = [];

    #[Groups(['command-task:read'])]
    public array $commandGroups = [];

    #[Groups(['command-task:read'])]
    public array $clients = [];

    #[Groups(['command-task:read'])]
    public \DateTimeInterface $dateTime;

    #[Groups(['command-task:read'])]
    public ?string $notes = null;

    #[Groups(['command-task:read'])]
    public ?string $status = null;

    #[Groups(['command-task:read'])]
    public \DateTime $createdAt;

    #[Groups(['command-task:read'])]
    public ?string $createdBy = null;


    public function __construct(CommandTask $commandTask)
    {
        parent::__construct($commandTask);

        $this->commands = $commandTask->getCommands()->map(
            fn(Command $command) => new CommandOutput($command)
        )->toArray();

        $this->commandGroups = $commandTask->getCommandGroups()->map(
            fn(CommandGroup $commandGroup) => new CommandGroupOutput($commandGroup)
        )->toArray();

        $this->clients = $commandTask->getClients()->map(
            fn(Client $client) => new ClientOutput($client)
        )->toArray();

        $this->dateTime = $commandTask->getDateTime();
        $this->notes = $commandTask->getNotes();
        $this->status = $commandTask->getStatus();
        $this->createdAt = $commandTask->getCreatedAt();
        $this->createdBy = $commandTask->getCreatedBy();
    }
}