<?php

namespace App\Dto\Output;

use ApiPlatform\Metadata\ApiProperty;
use ApiPlatform\Metadata\Get;
use App\Entity\OrganizationalUnit;
use App\Entity\Subnet;
use Symfony\Component\Serializer\Annotation\Groups;

#[Get(shortName: 'Subnet')]
final class SubnetOutput extends AbstractOutput
{
    #[Groups(['subnet:read'])]
    public string $name;

    #[Groups(['subnet:read'])]
    public string $netmask;

    #[Groups(['subnet:read'])]
    public string $ipAddress;

    #[Groups(['subnet:read'])]
    public string $nextServer;

    #[Groups(['subnet:read'])]
    public string $bootFileName;

    #[Groups(['subnet:read'])]
    public array $organizationalUnits;

    #[Groups(['subnet:read'])]
    public \DateTime $createdAt;

    #[Groups(['subnet:read'])]
    public ?string $createdBy = null;

    public function __construct(Subnet $subnet)
    {
        parent::__construct($subnet);

        $this->name = $subnet->getName();
        $this->netmask = $subnet->getNetmask();
        $this->ipAddress = $subnet->getIpAddress();
        $this->nextServer = $subnet->getNextServer();
        $this->bootFileName = $subnet->getBootFileName();


        $this->organizationalUnits = $subnet->getOrganizationalUnits()->map(
            fn(OrganizationalUnit $organizationalUnit) => new OrganizationalUnitOutput($organizationalUnit)
        )->toArray();

        $this->createdAt = $subnet->getCreatedAt();
        $this->createdBy = $subnet->getCreatedBy();
    }
}