<?php

namespace App\Service\OgBoot;

use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;
use Symfony\Component\HttpClient\HttpClient;


readonly class StatusService
{
    public function __construct(
        private string              $ogBootApiUrl
    )
    {
    }

    /**
     * @throws TransportExceptionInterface
     * @throws ServerExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ClientExceptionInterface
     */
    public function __invoke()
    {
        $httpClient = HttpClient::create([
            'verify_peer' => false, // Ignorar la verificación del certificado SSL
            'verify_host' => false, // Ignorar la verificación del nombre del host
        ]);

        try {
            $response = $httpClient->request('GET', $this->ogBootApiUrl.'/ogboot/v1/status', [
                'headers' => [
                    'accept' => 'application/json',
                ],
            ]);
        } catch (TransportExceptionInterface $e) {
            return new JsonResponse( data: 'An error occurred', status: Response::HTTP_INTERNAL_SERVER_ERROR);
        }

        return json_decode($response->getContent(), true);
    }
}