<?php

namespace App\State\Processor;

use ApiPlatform\Metadata\Delete;
use ApiPlatform\Metadata\Operation;
use ApiPlatform\Metadata\Patch;
use ApiPlatform\Metadata\Post;
use ApiPlatform\Metadata\Put;
use ApiPlatform\State\ProcessorInterface;
use ApiPlatform\Validator\ValidatorInterface;
use App\Dto\Input\OgLiveInput;
use App\Dto\Output\OgLiveOutput;
use App\Repository\OgLiveRepository;

readonly class OgLiveProcessor implements ProcessorInterface
{
    public function __construct(
        private OgLiveRepository            $ogLiveRepository,
        private ValidatorInterface          $validator
    )
    {
    }

    /**
     * @throws \Exception
     */
    public function process(mixed $data, Operation $operation, array $uriVariables = [], array $context = []): OgLiveOutput|null
    {
        switch ($operation){
            case $operation instanceof Post:
            case $operation instanceof Put:
            case $operation instanceof Patch:
                return $this->processCreateOrUpdate($data, $operation, $uriVariables, $context);
            case $operation instanceof Delete:
                return $this->processDelete($data, $operation, $uriVariables, $context);
        }
    }

    /**
     * @throws \Exception
     */
    private function processCreateOrUpdate($data, Operation $operation, array $uriVariables = [], array $context = []): OgLiveOutput
    {
        if (!($data instanceof OgLiveInput)) {
            throw new \Exception(sprintf('data is not instance of %s', OgLiveInput::class));
        }

        $entity = null;
        if (isset($uriVariables['uuid'])) {
            $entity = $this->ogLiveRepository->findOneByUuid($uriVariables['uuid']);
        }

        $ogLive = $data->createOrUpdateEntity($entity);
        $this->validator->validate($ogLive);
        $this->ogLiveRepository->save($ogLive);

        return new OgLiveOutput($ogLive);
    }

    private function processDelete($data, Operation $operation, array $uriVariables = [], array $context = []): null
    {
        $ogLive = $this->ogLiveRepository->findOneByUuid($uriVariables['uuid']);
        $this->ogLiveRepository->delete($ogLive);

        return null;
    }
}
