<?php

namespace App\State\Processor;

use ApiPlatform\Metadata\Delete;
use ApiPlatform\Metadata\Operation;
use ApiPlatform\Metadata\Patch;
use ApiPlatform\Metadata\Post;
use ApiPlatform\Metadata\Put;
use ApiPlatform\State\ProcessorInterface;
use ApiPlatform\Validator\ValidatorInterface;
use App\Dto\Input\PxeBootFileInput;
use App\Dto\Output\PxeBootFileOutput;
use App\Repository\PxeBootFileRepository;
use App\Service\OgBoot\PxeBootFile\PostService;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;

readonly class PxeBootFileProcessor implements ProcessorInterface
{
    public function __construct(
        private PxeBootFileRepository            $bootFileRepository,
        private ValidatorInterface               $validator,
        private PostService                      $postService
    )
    {
    }

    /**
     * @throws \Exception
     */
    public function process(mixed $data, Operation $operation, array $uriVariables = [], array $context = []): PxeBootFileOutput|null
    {
        switch ($operation){
            case $operation instanceof Post:
            case $operation instanceof Put:
            case $operation instanceof Patch:
                return $this->processCreateOrUpdate($data, $operation, $uriVariables, $context);
            case $operation instanceof Delete:
                return $this->processDelete($data, $operation, $uriVariables, $context);
        }
    }

    /**
     * @throws \Exception
     * @throws TransportExceptionInterface
     */
    private function processCreateOrUpdate($data, Operation $operation, array $uriVariables = [], array $context = []): PxeBootFileOutput
    {
        if (!($data instanceof PxeBootFileInput)) {
            throw new \Exception(sprintf('data is not instance of %s', PxeBootFileInput::class));
        }

        $entity = null;
        if (isset($uriVariables['uuid'])) {
            $entity = $this->bootFileRepository->findOneByUuid($uriVariables['uuid']);
        }

        $pxeTemplate = $data->createOrUpdateEntity($entity);
        $this->validator->validate($pxeTemplate);
        $this->bootFileRepository->save($pxeTemplate);

        try {
            $this->postService->__invoke($pxeTemplate);
        } catch (\Exception $e) {

        }

        return new PxeBootFileOutput($pxeTemplate);
    }

    private function processDelete($data, Operation $operation, array $uriVariables = [], array $context = []): null
    {
        $bootFile = $this->bootFileRepository->findOneByUuid($uriVariables['uuid']);
        $this->bootFileRepository->delete($bootFile);

        return null;
    }
}
